@@ -17,8 +17,7 @@ score.delta = function(arc, network, data, score, score.delta,
                         nodes = arc, extra.args = extra, data = data)
 
     # update the test counter.
-    assign(".test.counter", get(".test.counter", envir = .GlobalEnv) + 2,
-      envir = .GlobalEnv)
+    increment.test.counter(2)
 
     # compare the network scores, minus numeric tolerance for better score
     # equivalence detection.
@@ -35,8 +34,7 @@ score.delta = function(arc, network, data, score, score.delta,
                         nodes = arc[2], extra.args = extra, data = data)
 
     # update the test counter.
-    assign(".test.counter", get(".test.counter", envir = .GlobalEnv) + 1,
-      envir = .GlobalEnv)
+    increment.test.counter(1)
 
     # compare the network scores.
     retval = new.score - reference.score[arc[2]]
