@@ -160,21 +160,21 @@ inter.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklis
   if (!is.null(backtracking)) {
 
     # nodes whose markov blanket includes this node are included, because
-    # X \in MB(Y) <=> Y \in MB(X)
+    # X \in MB(Y) <=> Y \in MB(X); they can be removed in the backward phase
+    # later on if they are false positives.
+
     known.good = names(backtracking[backtracking])
     mb = unique(c(mb, known.good))
 
     # and vice versa X \not\in MB(Y) <=> Y \not\in MB(X)
     known.bad = names(backtracking[!backtracking])
 
-    # both are not to be checked for inclusion/exclusion.
-    nodes = nodes[!(nodes %in% names(backtracking))]
-
     if (debug) {
 
       cat("    * known good (backtracking): '", known.good, "'.\n")
       cat("    * known bad (backtracking): '", known.bad, "'.\n")
-      cat("    * nodes still to be tested for inclusion: '", nodes, "'.\n")
+      cat("    * nodes still to be tested for inclusion: '",
+        nodes[!(nodes %in% mb)], "'.\n")
 
     }#THEN
 
@@ -209,12 +209,12 @@ inter.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklis
 
     if (association[to.add] <= alpha) mb = c(mb, to.add)
 
-    # whitelisted nodes are neighbours, they cannot be removed from the
-    # markov blanket; the last node added in phase I will never be removed,
-    # because the tests for inclusion and removal are identical.
-    # known.good nodes from backtracking are not to be removed, either.
+    # whitelisted nodes are neighbours, they cannot be removed from the markov
+    # blanket; the last node added in phase I will never be removed, because
+    # the tests for inclusion and removal are identical; on the other hand,
+    # known.good nodes should be checked to remove false positives.
     if (length(mb) > 1)
-      sapply(mb[!(mb %in% c(known.good, to.add, whitelisted))], del.node, x = x, test = test)
+      sapply(mb[!(mb %in% c(to.add, whitelisted))], del.node, x = x, test = test)
 
     if (identical(mb, mb.snapshot)) {
 
@@ -265,7 +265,7 @@ inter.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklis
 
   }#REPEAT
 
-  mb
+  return(mb)
 
 }#INTER.IA.MARKOV.BLANKET
 
