@@ -37,7 +37,7 @@ maxmin.pc.cluster = function(x, cluster, whitelist, blacklist,
   # 1. [Forward Phase (I)]
   mb = parLapply(cluster, as.list(nodes), maxmin.pc.forward.phase, data = x,
          nodes = nodes, alpha = alpha, B = B, whitelist = whitelist,
-         blacklist = blacklist, test = test, optimized = TRUE, debug = debug)
+         blacklist = blacklist, test = test, optimized = FALSE, debug = debug)
   names(mb) = nodes
 
   # 2. [Backward Phase (II)]
@@ -61,7 +61,7 @@ maxmin.pc = function(x, whitelist, blacklist, test, alpha, B,
   # 1. [Forward Phase (I)]
   mb = lapply(as.list(nodes), maxmin.pc.forward.phase, data = x, nodes = nodes,
          alpha = alpha, B = B, whitelist = whitelist, blacklist = blacklist,
-         test = test, optimized = TRUE, debug = debug)
+         test = test, optimized = FALSE, debug = debug)
   names(mb) = nodes
 
   # 2. [Backward Phase (II)]
@@ -105,21 +105,25 @@ maxmin.pc.forward.phase = function(x, data, nodes, alpha, B, whitelist,
   # use backtracking for a further screening of the nodes to be checked.
   if (!is.null(backtracking) && optimized) {
 
-    # X adiacent to Y <=> Y adiacent to X
+    # nodes whose neighbourhood includes this node are included, because
+    # X \in NBR(Y) <=> Y \in NBR(X); they can be removed in the backward
+    # phase later on if they are false positives.
     known.good = names(backtracking[backtracking])
     cpc = unique(c(cpc, known.good))
 
     # and vice versa X not adiacent to Y <=> Y not adiacent to X
     known.bad = names(backtracking[!backtracking])
 
-    # both are not to be checked for inclusion/exclusion.
-    nodes = nodes[!(nodes %in% names(backtracking))]
+    # from empirical observations, MMPC has very few false negatives and
+    # therefore it is fairly safe not to test known.bad nodes.
+    nodes  = nodes[!(nodes %in% known.bad)] 
 
     if (debug) {
 
       cat("    * known good (backtracking): '", known.good, "'.\n")
       cat("    * known bad (backtracking): '", known.bad, "'.\n")
-      cat("    * nodes still to be tested for inclusion: '", nodes, "'.\n")
+      cat("    * nodes still to be tested for inclusion: '",
+        nodes[!(nodes %in% cpc)], "'.\n")
 
     }#THEN
 
@@ -128,26 +132,14 @@ maxmin.pc.forward.phase = function(x, data, nodes, alpha, B, whitelist,
   # phase I (stepwise forward selection)
   repeat {
 
+    # do not check nodes which have a p-value above the alpha threshold, as
+    # it can only increase; also do not check 'known bad' ones.
+    to.be.checked = setdiff(names(which(association < alpha)), c(cpc, known.bad))
+
     # get an association measure for each of the available nodes.
-    if (optimized) {
-
-      # do not check nodes which have a p-value above the alpha
-      # threshold, as it can only increase; do not check both 'known
-      # bad' and 'known good' ones.
-      to.be.checked = setdiff(names(which(association < alpha)), c(cpc, known.bad))
-
-      association = sapply(to.be.checked, maxmin.pc.heuristic.optimized, y = x,
-                      sx = cpc, data = data, test = test, alpha = alpha, B = B,
-                      association = association, debug = debug)
-
-    }#THEN
-    else {
-
-      association = sapply(nodes, maxmin.pc.heuristic, y = x, sx = cpc,
-                      data = data, test = test, alpha = alpha, B = B,
-                      debug = debug)
-
-    }#ELSE
+    association = sapply(to.be.checked, maxmin.pc.heuristic.optimized, y = x,
+                    sx = cpc, data = data, test = test, alpha = alpha, B = B,
+                    association = association, debug = debug)
 
     # stop if there are no candidates for inclusion.
     if (all(association > alpha) || length(nodes) == 0 || is.null(nodes)) break
@@ -175,52 +167,6 @@ maxmin.pc.forward.phase = function(x, data, nodes, alpha, B, whitelist,
 
 }#MAXMIN.PC.FORWARD.PHASE
 
-maxmin.pc.heuristic = function(x, y, sx, data, test, alpha, B, debug = FALSE) {
-
-  k = 0
-  min.assoc = 0
-
-  if (debug)
-    cat("  * checking node", x ,"for association.\n")
-
-  repeat {
-
-    # create all the possible subsets of size k of the candidate
-    # parent-children set.
-    dsep.subsets = subsets(length(sx), k, sx)
-
-    for (s in 1:nrow(dsep.subsets)) {
-
-      a = conditional.test(x, y, dsep.subsets[s,], data = data, test = test, B = B,
-            alpha = alpha)
-
-      if (debug) {
-
-        cat("    > trying conditioning subset '", dsep.subsets[s,], "'.\n")
-        cat("    > node", x, "has p-value:", a, ".\n")
-
-      }#THEN
-
-      # minimum association means maximum p-value.
-      min.assoc = max(min.assoc, a)
-
-    }#FOR
-
-    if (k < length(sx))
-      k = k + 1
-    else
-      break
-
-  }#REPEAT
-
-  if (debug)
-    cat("    > node", x, "has a minimum association of",
-              min.assoc, ".\n")
-
-  return(min.assoc)
-
-}#MAXMIN.PC.HEURISTIC
-
 maxmin.pc.heuristic.optimized = function(x, y, sx, data, test, alpha, B,
     association, debug = FALSE) {
 
