@@ -10,3 +10,13 @@ arcs2elist = function(arcs, nodes, id = TRUE, sublist = TRUE) {
         PACKAGE = "bnlearn")
 
 }#ARCS2ELIST
+
+# convert an edge list into an arc set.
+elist2arcs = function(elist) {
+
+  .Call("elist2arcs",
+        elist = elist,
+        PACKAGE = "bnlearn")
+
+}#ELIST2ARCS
+
