@@ -24,7 +24,7 @@ mb2arcs = function(mb, nodes) {
   result = do.call(rbind, lapply(nodes[!empty.mb],
                function(x) { cbind(from = x, to = mb[[x]][['nbr']]) }))
 
-  # return an empty matrix all markov blankets are empty.
+  # return an empty matrix if all markov blankets are empty.
   if (is.null(result))
     matrix(character(0), ncol = 2, dimnames = list(c(), c("from", "to")))
   else
@@ -69,6 +69,7 @@ mb.fitted = function(x, node) {
 # the condition \sum \pi_{i} = 1).
 nparams.discrete = function(x, data, real = FALSE, debug = FALSE) {
 
+  # works for both dnode and onode objects, they have the same structure.
   .Call("nparams_dnet",
         graph = x,
         data = data,
@@ -191,6 +192,85 @@ cpdag.arc.backend = function(nodes, arcs, moral = FALSE, fix.directed = FALSE,
 
 }#CPDAG.ARCS.BACKEND
 
+# mutilated network graph used in likelihood weighting.
+mutilated.backend.bn = function(x, evidence) {
+
+  # this is basically a NOP.
+  if (identical(evidence, TRUE))
+    return(x) 
+
+  # only the node names are used here.
+  fixed = names(evidence)
+  nodes = names(x$nodes)
+  # remove all parents of nodes in the evidence.    
+  x$arcs = x$arcs[!(x$arcs[, "to"] %in% fixed), ]
+  # update the cached information for the fixed nodes.
+  amat = arcs2amat(x$arcs, nodes)
+  for (node in fixed)
+    x$nodes[[node]] = cache.partial.structure(nodes, target = node,
+                        amat = amat, debug = FALSE)
+
+  return(x)
+
+}#MUTILATED.BACKEND.BN
+
+# mutilated fitted network used in likelihood sampling.
+mutilated.backend.fitted = function(x, evidence) {
+
+  # this is basically a NOP.
+  if (identical(evidence, TRUE))
+    return(x) 
+
+  # extract the names of the nodes.
+  fixed = names(evidence)
+  nodes = names(x)
+      
+  for (node in fixed) {
+
+    # cache the node information.
+    cur = x[[node]]
+    fix = evidence[[node]]
+
+    if (is(cur, "bn.fit.gnode")) {
+
+      # reset the conditional distribution.
+      cur$coefficients = c("(Intercept)" = as.numeric(fix))
+      cur$sd = 0
+      # reset fitted values and residuals.
+      if (!is.null(cur$fitted.values))
+        cur$residuals = rep(fix, length(cur$fitted.values))
+      if (!is.null(cur$residuals))
+        cur$residuals = rep(0, length(cur$residuals))
+
+    }#THEN
+    else if(is(cur, c("bn.fit.dnode", "bn.fit.onode"))) {
+
+      # reset the conditional distribution.
+      cur$prob = as.table(structure((dimnames(cur$prob)[[1]] == fix) + 0, 
+                   names = dimnames(cur$prob)[[1]]))
+
+    }#THEN
+
+    # update parents and children.
+    parents = cur$parents
+    cur$parents = character(0)
+
+    for (p in parents) {
+
+      temp = x[[p]]
+      temp$children = temp$children[temp$children != node]
+      x[p] = list(temp)
+
+    }#FOR
+
+    x[node] = list(cur)
+
+  }#FOR
+
+  return(x)
+
+}#MUTILATED.BACKEND.FITTED
+
 # apply random arc operators to the graph.
 perturb.backend = function(network, iter, nodes, amat, whitelist,
     blacklist, debug = FALSE) {
